/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.internal.common.SelectionWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");

var CommonMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var Util					= com.ibm.rational.report.web.core.app.internal.Util;

dojo.declare(
	"com.ibm.rational.report.web.core.app.internal.common.SelectionWidget",
	[dijit._Widget, dijit._Templated], {
	
	// args: availableOptions, selectedOptions, availableItemsTitle, selectedItemsTitle
	// map of option's value to the option {value:value, text:text}
	_optionsMap: [],
	
	// save the initial selected options for keep track of the changes
	_initialSelected: [],
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/templates/SelectionWidget.html"),
	
	constructor: function(args) {
		this._args = args;
	},
	
	postCreate: function() {
		this._optionsMap = [];
		this._initialSelected = [];
		
		var self = this;		
		var args = this._args;
		
		var leftToRightBtnText;
		var rightToLeftBtnText;
		
		if (args.leftToRightBtnText != null && typeof args.leftToRightBtnText != "undefined") {
			Util.removeChildren(this.addButton);
			this.addButton.appendChild(document.createTextNode(args.leftToRightBtnText));
		}
		this.addButton.title = CommonMessages.ADD_BUTTON_TOOLTIP;

		if (args.rightToLeftBtnText != null && typeof args.rightToLeftBtnText != "undefined") {
			Util.removeChildren(this.removeButton);
			this.removeButton.appendChild(document.createTextNode(args.rightToLeftBtnText));
		}
		this.removeButton.title = CommonMessages.REMOVE_BUTTON_TOOLTIP;		
				
		if (args.selectedOptions != null && typeof args.selectedOptions != "undefined") {
			for (var i in args.selectedOptions) {
				var opt = args.selectedOptions[i];
				this._optionsMap[opt.value] = opt;
			}
		}
			
		if (args.availableOptions != null && typeof args.availableOptions != "undefined") {
			for (var i in args.availableOptions) {
				var opt = args.availableOptions[i];
				this._optionsMap[opt.value] = opt;
			}		
		}
		
		var availableItemsTitle;
		if (args.availableItemsTitle != null && typeof args.availableItemsTitle != "undefined") {
			availableItemsTitle = args.availableItemsTitle;
		}
		else {
			availableItemsTitle = "Available Items:";
		}
		this.availableItemsTitle.appendChild(document.createTextNode(availableItemsTitle));		
		
		var selectedItemsTitle;
		if (args.selectedItemsTitle != null && typeof args.selectedItemsTitle != "undefined") {
			selectedItemsTitle = args.selectedItemsTitle;
		}
		else {
			selectedItemsTitle = "Selected Items:";
		}			
		this.selectedItemsTitle.appendChild(document.createTextNode(selectedItemsTitle));	
				
		this.fillControls(args.selectedOptions);
				
		this._readOnly = false;
		if (args.readOnly != null ) {
			this._readOnly = args.readOnly;
		}
		
		if (this._readOnly) {	
			this.addButton.disabled = "disabled";
			this.removeButton.disabled = "disabled";
		}
		
		// save and cancel actions
		dojo.connect(this.addButton, "onclick",
			function(e) {
				e.preventDefault();
				self.addButton.disabled = true;
				self._handleAdd();
			}
		);
		
		dojo.connect(this.availableControl, "onchange",
			function(e) {
				e.preventDefault();
				self.addButton.disabled = false;
			}
		);
		
		dojo.connect(this.removeButton, "onclick",
			function(e) {
				e.preventDefault();
				self.removeButton.disabled = true;
				self._handleRemove();
			}
		);		
		
		dojo.connect(this.selectedControl, "onchange",
			function(e) {
				e.preventDefault();
				self.removeButton.disabled = false;
			}
		);
		
	},
	
	_addOption: function(ctrl, data) {
		ctrl.options[ctrl.options.length] = new Option(data.text, data.value, false, false);	
	},
	
	_copyOption: function(opt) {
		return new Option(opt.text, opt.value, false, false);	
	},
	
	_handleAdd: function() {
		this._moveOption(this.availableControl, this.selectedControl);
	},
	
	_handleRemove: function() {
		this._moveOption(this.selectedControl, this.availableControl);	
	},
	
	_moveOption: function(from, to) {	
		var i = 0;
		while ( i < from.options.length ) {
			var opt = from.options[i];
			if ( opt.selected ) {
			
				// IE does not allow assign an option object from one selection list to another
				// so we need to make a new option and copy the attributes over
				// as a result, the old option should be removed from the source list
				//if ( ) {
					to.options[to.options.length] = this._copyOption(opt);
					from.remove(i);			
				//} else {
					//to.options[to.options.length] = opt;
					// remove the option from the from list
					// assign the option to another control will remove from the current control automatically
					// so no need to call remove
				//}
			} else {
				i++;
			}
		}

	},
	
	fillControls: function(selectedOptions) {
		if (selectedOptions == null || typeof selectedOptions == "undefined") {
			selectedOptions = [];
		}
		
		this.selectedControl.options.length = 0;
		this.availableControl.options.length = 0;
		this._initialSelected = [];
		
		for (var i in selectedOptions) {
			var opt = selectedOptions[i];
			if (typeof opt.value == "undefined") {
				opt = this._optionsMap[opt];
			}
			
			this._initialSelected[opt.value] = opt;
			this._addOption(this.selectedControl, opt);
		}
		
		for (var i in this._optionsMap) {
			var opt = this._optionsMap[i];
					
			if (!this._initialSelected[opt.value]) {
				this._addOption(this.availableControl, opt);
			}
		}		
	
	},
	
	getAllSelected: function() {
		var result = [];
		for (var i = 0; i < this.selectedControl.options.length; i++) {
			var opt = this.selectedControl.options[i];	
			result.push(this._optionsMap[opt.value]);
		}
		
		return result;
	},
	
	getAdded: function() {
		var result = [];
		for (var i = 0; i < this.selectedControl.options.length; i++) {
			var opt = this.selectedControl.options[i];
			if (!this._initialSelected[opt.value]) {
				result.push(this._optionsMap[opt.value]);
			}
		}
		
		return result;
	},
	
	getRemoved: function() {
		var selected = [];
		for (var i = 0; i < this.selectedControl.options.length; i++) {
			var opt = this.selectedControl.options[i];
			selected[opt.value] = true;
		}
				
		var result = [];
		for (var v in this._initialSelected) {
			var opt = this._initialSelected[v];
			if (!selected[v]) {
				result.push(opt);
			}
		}
		return result;
	},
	
	__END: true
});

})();
